#!/usr/bin/env python3
import datetime
import json
import os
from base64 import urlsafe_b64encode
from hashlib import sha256
from secrets import token_urlsafe
from urllib.parse import urlencode
from pathlib import Path

import requests
from pixivpy3 import *

from pixivd.AESCipher import AESCipher
from pixivd.i18n import i18n as _


class Pixiv_Get_Error(Exception):
    def __init__(self, url, Err=None):
        self.url = url
        self.error = Err

    def __str__(self):
        return 'Failed to get data: ' + self.url


class PixivApi:
    """
    Attribution:
        access_token
        user_id: str, login user id
        User_Agent: str, the version of pixiv app
    """
    aapi = AppPixivAPI()
    user_agent = 'PixivIOSApp/6.4.0'
    hash_secret = '28c1fdd170a5204386cb1313c7077b34f83e4aaf4aa829ce78c231e05b0bae2c'
    session = ''
    user_id = ''
    image_sizes = ','.join(['px_128x128', 'px_480mw', 'small', 'medium', 'large'])
    profile_image_sizes = ','.join(['px_170x170', 'px_50x50'])
    timeout = 20
    access_token = ''
    refresh_token = ''
    client_id = 'MOBrBDS8blbauoSck0ZfDbtuzpyT'
    client_secret = 'lsACyCD94FhDUtGTXi3QzcFE2uU1hqtDaKeqrdwj'
    auth_token_url = 'https://oauth.secure.pixiv.net/auth/token'
    session_path = Path(__file__).parent / 'data' / 'session'

    def __init__(self):
        self.ensure_session_dir()
        self.login_required()

    def ensure_session_dir(self):
        if not Path(self.session_path).parent.exists():
            self.session_path.parent.mkdir()

    def load_session(self):
        cipher = AESCipher()
        with open(self.session_path, 'rb') as f:
            enc = f.read()
        try:
            plain = cipher.decrypt(enc)
            loaded_session = json.loads(str(plain))
            self.access_token = loaded_session['access_token']
            self.refresh_token = loaded_session['refresh_token']
            return True
        except:
            print("error when load session, please delete session file and try again.")

    def save_session(self):
        data = {
            'access_token': self.access_token,
            'refresh_token': self.refresh_token
        }
        cipher = AESCipher()
        enc = cipher.encrypt(json.dumps(data))
        with open(self.session_path, 'wb') as f:
            f.write(enc)

    def parse_token(self, data):
        return data["access_token"], data["refresh_token"]

    def login(self):
        """
        logging to Pixiv
        doc: https://gist.github.com/ZipFile/c9ebedb224406f4f11845ab700124362

        Return:
            a requests session object

        """
        REDIRECT_URI = "https://app-api.pixiv.net/web/v1/users/auth/pixiv/callback"
        LOGIN_URL = "https://app-api.pixiv.net/web/v1/login"

        def s256(data):
            """S256 transformation method."""
            return urlsafe_b64encode(sha256(data).digest()).rstrip(b"=").decode("ascii")

        def oauth_pkce(transform):
            """Proof Key for Code Exchange by OAuth Public Clients (RFC7636)."""

            code_verifier = token_urlsafe(32)
            code_challenge = transform(code_verifier.encode("ascii"))

            return code_verifier, code_challenge

        code_verifier, code_challenge = oauth_pkce(s256)
        login_params = {
            "code_challenge": code_challenge,
            "code_challenge_method": "S256",
            "client": "pixiv-android",
        }

        print(
            f"Please open {LOGIN_URL}?{urlencode(login_params)}\n, and following steps in https://gist.github.com/ZipFile/c9ebedb224406f4f11845ab700124362 to get code")

        try:
            code = input("Please input code: ").strip()
        except (EOFError, KeyboardInterrupt):
            return

        r = requests.post(
            self.auth_token_url,
            data={
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                "code": code,
                "code_verifier": code_verifier,
                "grant_type": "authorization_code",
                "include_policy": "true",
                "redirect_uri": REDIRECT_URI,
            },
            headers={"User-Agent": self.user_agent},
        )
        self.access_token, self.refresh_token = self.parse_token(r.json())
        print(f'refresh token: {self.refresh_token}')
        self.refresh()

    def refresh(self):
        """Use refresh token to get new access token"""
        # self.aapi.auth(refresh_token='cKwCinT4vVbRy4kOitoQTA7Q1lhjBr69tGy44fI-3Ho')
        self.aapi.auth(refresh_token=self.refresh_token)
        self.access_token, self.refresh_token = self.aapi.access_token, self.aapi.refresh_token
        self.save_session()

    def login_required(self):
        if self.session_path.exists():
            if self.load_session():
                self.refresh()

        if not self.access_token:
            print(_('Please login'))
            self.login()

    def get_all_user_illustrations(self, user_id, offset=0, size=-1):
        """

        :param user_id: str
        :param offset: int
        :param size: int, max result length, if < 0, return all
        :return:
         [
            {
              "id": 92990893,
              "title": "-",
              "type": "illust",
              "image_urls": {
                "square_medium": "https://i.pximg.net/c/360x360_70/img-master/img/2021/09/25/00/05/35/92990893_p0_square1200.jpg",
                "medium": "https://i.pximg.net/c/540x540_70/img-master/img/2021/09/25/00/05/35/92990893_p0_master1200.jpg",
                "large": "https://i.pximg.net/c/600x1200_90/img-master/img/2021/09/25/00/05/35/92990893_p0_master1200.jpg"
              },
              "caption": "",
              "restrict": 0,
              "user": {
                "id": 22124330,
                "name": "\u8d85\u51f6\u306e\u72c4\u7490\u5361",
                "account": "swd3e22",
                "profile_image_urls": {
                  "medium": "https://i.pximg.net/user-profile/img/2017/01/10/13/28/42/11988991_bae951a38d31d217fa1eceedc0aafdbe_170.jpg"
                },
                "is_followed": true
              },
              "tags": [
                {
                  "name": "\u5973\u306e\u5b50",
                  "translated_name": "girl"
                },
                {
                  "name": "\u843d\u66f8",
                  "translated_name": "doodle"
                },
                {
                  "name": "closers",
                  "translated_name": null
                },
                {
                  "name": "\u30b7\u30e7\u30fc\u30c8\u30d1\u30f3\u30c4",
                  "translated_name": "short pants"
                },
                {
                  "name": "\u9b45\u60d1\u306e\u9854",
                  "translated_name": "alluring face"
                },
                {
                  "name": "Mirae",
                  "translated_name": null
                },
                {
                  "name": "\u80f8\u30dd\u30c1",
                  "translated_name": "hanging breasts"
                },
                {
                  "name": "\u898b\u305b\u30cf\u30a4\u30ec\u30b0\u30d1\u30f3\u30c4/\u30c1\u30e7\u30fc\u30ab\u30fc/\u30a2\u30fc\u30e0\u30ab\u30d0\u30fc/\u30cf\u30fc\u30cd\u30b9",
                  "translated_name": null
                },
                {
                  "name": "\u7740\u8863\u5de8\u4e73/\u80f8\u306b\u624b/\u30a2\u30e1\u30ea\u30ab\u30f3\u30b9\u30ea\u30fc\u30d6/\u3078\u305d\u51fa\u3057/\u307a\u305f\u3093\u5ea7\u308a",
                  "translated_name": null
                },
                {
                  "name": "\u30c1\u30e3\u30c3\u30af\u4e0b\u308d\u3057/\u6c57/\u64ab\u3067\u56de\u3057\u305f\u3044\u304a\u8179/\u3082\u3082\u3077\u304f",
                  "translated_name": null
                }
              ],
              "tools": [],
              "create_date": "2021-09-25T00:05:35+09:00",
              "page_count": 1,
              "width": 1329,
              "height": 1919,
              "sanity_level": 4,
              "x_restrict": 0,
              "series": null,
              "meta_single_page": {
                "original_image_url": "https://i.pximg.net/img-original/img/2021/09/25/00/05/35/92990893_p0.jpg"
              },
              "meta_pages": [],
              "total_view": 45615,
              "total_bookmarks": 8179,
              "is_bookmarked": false,
              "visible": true,
              "is_muted": false,
              "total_comments": 28
            }
         ]

        """

        r = []
        done = False
        cur_size = 0

        while not done:
            data = self.aapi.user_illusts(user_id, offset=offset)
            try:
                r.extend(data['illusts'])
            except:
                print(data)
            offset += 30
            cur_size += 30
            if not data['next_url'] or (0 <= size <= cur_size):
                done = True
        return r[:size] if size >= 0 else r

    def get_illustration(self, illustration_id):
        """
        get illustration data

        Return: list of illustration instances
        """
        self.login_required()
        return [self.aapi.illust_detail(illustration_id)['illust']]

    def get_ranking_illustrations(self, mode='day', date='', total_page=3):
        """
        fetch illustrations by ranking

        Args:
            mode: [day, week, month, day_male, day_female, week_original, week_rookie, day_manga,
                   day_r18, day_male_r18, day_female_r18, week_r18, week_r18g]
            date: '2015-04-01'
            per_page: int
            page: int

        Return: list of illustration data, same as `get_all_user_illustrations()`
        """
        self.login_required()
        r = []
        page = 0
        offset = 0
        while page <= total_page:
            data = self.aapi.illust_ranking(mode, date=date, offset=offset)
            r.extend(data['illusts'])
            offset += 30
            page += 1
        for index, i in enumerate(r):
            i['rank'] = index + 1
        return r

    def set_timeout(self, timeout):
        self.timeout = timeout



#!/usr/bin/env python3
"""
pixiv

Usage:
    pixivd
    pixivd <id>...
    pixivd -r [-d | --date=<date>]
    pixivd -u

Arguments:
    <id>                                       user_ids

Options:
    -r                                         Download by ranking
    -d <date> --date <date>                    Target date
    -u                                         Update exist folder
    -h --help                                  Show this screen
    -v --version                               Show version

Examples:
    pixivd 7210261 1980643
    pixivd -r -d 2016-09-24
"""
import datetime
import math
import os
import queue
import re
import sys
import threading
import time
import traceback

import requests
from docopt import docopt
from tqdm import tqdm

from pixivd.api import PixivApi
from pixivd.i18n import i18n as _
from pixivd.model import PixivIllustModel

_THREADING_NUMBER = 10
_finished_download = 0
_CREATE_FOLDER_LOCK = threading.Lock()
_PROGRESS_LOCK = threading.Lock()
_SPEED_LOCK = threading.Lock()
_Global_Download = 0
_error_count = {}
_ILLUST_PER_PAGE = 30
_MAX_ERROR_COUNT = 5

__version__ = '3.0'


def get_default_save_path():
    current_path = os.getcwd()
    filepath = os.path.join(current_path, 'illustrations')
    if not os.path.exists(filepath):
        with _CREATE_FOLDER_LOCK:
            if not os.path.exists(os.path.dirname(filepath)):
                os.makedirs(os.path.dirname(filepath))
            os.makedirs(filepath)
    return filepath


def get_speed(elapsed):
    """Get current download speed"""
    with _SPEED_LOCK:
        global _Global_Download
        down = _Global_Download
        _Global_Download = 0
    speed = down / elapsed
    if speed == 0:
        return '%8.2f /s' % 0
    units = [' B', 'KB', 'MB', 'GB', 'TB', 'PB']
    unit = math.floor(math.log(speed, 1024.0))
    speed /= math.pow(1024.0, unit)
    return '%6.2f %s/s' % (speed, units[unit])


def print_progress(max_size):
    global _finished_download
    pbar = tqdm(total=max_size)

    last = 0
    while _finished_download != max_size:
        pbar.update(_finished_download - last)
        last = _finished_download
        time.sleep(0.5)
    pbar.update(_finished_download - last)
    pbar.close()


def download_file(url, filepath):
    headers = {'Referer': 'http://www.pixiv.net/'}
    r = requests.get(url, headers=headers, stream=True, timeout=PixivApi.timeout)
    if r.status_code == requests.codes.ok:
        total_length = r.headers.get('content-length')
        if total_length:
            data = []
            for chunk in r.iter_content(1024 * 16):
                data.append(chunk)
                with _SPEED_LOCK:
                    global _Global_Download
                    _Global_Download += len(chunk)
            with open(filepath, 'wb') as f:
                list(map(f.write, data))
    else:
        raise ConnectionError('\r', _('Connection error: %s') % r.status_code)


def download_threading(download_queue):
    global _finished_download
    while not download_queue.empty():
        illustration = download_queue.get()
        filepath = illustration['path']
        filename = illustration['file']
        url = illustration['url']
        count = _error_count.get(url, 0)
        if count < _MAX_ERROR_COUNT:
            if not os.path.exists(filepath):
                with _CREATE_FOLDER_LOCK:
                    if not os.path.exists(os.path.dirname(filepath)):
                        os.makedirs(os.path.dirname(filepath))
                try:
                    download_file(url, filepath)
                    with _PROGRESS_LOCK:
                        _finished_download += 1
                except Exception as e:
                    if count < _MAX_ERROR_COUNT:
                        print(_('%s => %s download error, retry') % (e, filename))
                        download_queue.put(illustration)
                        _error_count[url] = count + 1
        else:
            print(url, 'reach max retries, canceled')
            with _PROGRESS_LOCK:
                _finished_download += 1
        download_queue.task_done()


def start_and_wait_download_threading(download_queue, count):
    """start download threading and wait till complete"""
    progress_t = threading.Thread(target=print_progress, args=(count,))
    progress_t.daemon = True
    progress_t.start()
    for i in range(_THREADING_NUMBER):
        download_t = threading.Thread(target=download_threading, args=(download_queue,))
        download_t.daemon = True
        download_t.start()

    progress_t.join()
    download_queue.join()


def get_filepath(url, illustration, save_path='.', add_user_folder=False, add_rank=False):
    """return (filename,filepath)"""

    if add_user_folder:
        user_id = illustration.user_id
        user_name = illustration.user_name
        current_path = get_default_save_path()
        cur_dirs = list(filter(os.path.isdir, [os.path.join(current_path, i) for i in os.listdir(current_path)]))
        cur_user_ids = [os.path.basename(cur_dir).split()[0] for cur_dir in cur_dirs]
        if user_id not in cur_user_ids:
            dir_name = re.sub(r'[<>:"/\\|\?\*]', ' ', user_id + ' ' + user_name)
        else:
            dir_name = list(i for i in cur_dirs if os.path.basename(i).split()[0] == user_id)[0]
        save_path = os.path.join(save_path, dir_name)

    filename = url.split('/')[-1]
    if add_rank:
        filename = f'{illustration.rank} - {filename}'
    filepath = os.path.join(save_path, filename)
    return filename, filepath


def check_files(illustrations, save_path='.', add_user_folder=False, add_rank=False):
    download_queue = queue.Queue()
    index_list = []
    count = 0
    if illustrations:
        last_i = -1
        for index, illustration in enumerate(illustrations):
            if not illustration.image_urls:
                continue
            else:
                for url in illustration.image_urls:
                    filename, filepath = get_filepath(url, illustration, save_path, add_user_folder, add_rank)
                    if os.path.exists(filepath):
                        continue
                    else:
                        if last_i != index:
                            last_i = index
                            index_list.append(index)
                        download_queue.put({'url': url, 'file': filename, 'path': filepath})
                        count += 1
    return download_queue, count, index_list


def count_illustrations(illustrations):
    return sum(len(i.image_urls) for i in illustrations)


def is_manga(illustrate):
    return True if illustrate.is_manga or illustrate.type == 'manga' else False


def download_illustrations(user, data_list, save_path='.', add_user_folder=False, add_rank=False, skip_manga=False):
    """Download illustratons

    Args:
        user: PixivApi()
        data_list: json
        save_path: str, download path of the illustrations
        add_user_folder: bool, whether put the illustration into user folder
        add_rank: bool, add illustration rank at the beginning of filename
    """
    illustrations = PixivIllustModel.from_data(data_list)
    if skip_manga:
        manga_number = sum([is_manga(i) for i in illustrations])
        if manga_number:
            print('skip', manga_number, 'manga')
            illustrations = list(filter(lambda x: not is_manga(x), illustrations))
    download_queue, count = check_files(illustrations, save_path, add_user_folder, add_rank)[0:2]
    if count > 0:
        print(_('Start download, total illustrations '), count)
        global _finished_download, _Global_Download
        _finished_download = 0
        _Global_Download = 0
        start_and_wait_download_threading(download_queue, count)
        print()
    else:
        print(_('There is no new illustration need to download'))


def download_by_user_id(user, user_ids=None):
    save_path = get_default_save_path()
    if not user_ids:
        user_ids = input(_('Input the artist\'s id:(separate with space)')).strip().split(' ')
    for user_id in user_ids:
        print(_('Artists %s') % user_id)
        data_list = user.get_all_user_illustrations(user_id)
        download_illustrations(user, data_list, save_path, add_user_folder=True)


def download_by_ranking(user):
    today = str(datetime.date.today())
    save_path = os.path.join(get_default_save_path(), today + ' ranking')
    data_list = user.get_ranking_illustrations()
    download_illustrations(user, data_list, save_path, add_rank=True)


def download_by_history_ranking(user, date=''):
    if not date:
        date = input(_('Input the date:(eg:2015-07-10)'))
    if not (re.search("^\d{4}-\d{2}-\d{2}", date)):
        print(_('[invalid date format]'))
        date = str(datetime.date.today() - datetime.timedelta(days=1))
    save_path = os.path.join(get_default_save_path(), date + ' ranking')
    data_list = user.get_ranking_illustrations(date=date)
    download_illustrations(user, data_list, save_path, add_rank=True)


def artist_folder_scanner(user, user_id_list, save_path, final_list, fast):
    while not user_id_list.empty():
        user_info = user_id_list.get()
        user_id = user_info['id']
        folder = user_info['folder']
        try:
            if fast:
                data_list = []
                offset = 0
                page_result = user.get_all_user_illustrations(user_id, offset, _ILLUST_PER_PAGE)
                if len(page_result) > 0:
                    data_list.extend(page_result)
                    file_path = os.path.join(save_path, folder, data_list[-1]['image_urls']['large'].split('/')[-1])
                    while not os.path.exists(file_path) and len(page_result) == _ILLUST_PER_PAGE:
                        offset += _ILLUST_PER_PAGE
                        page_result = user.get_all_user_illustrations(user_id, offset, _ILLUST_PER_PAGE)
                        data_list.extend(page_result)
                        file_path = os.path.join(save_path, folder, data_list[-1]['image_urls']['large'].split('/')[-1])
                        # prevent rate limit
                        time.sleep(1)
            else:
                data_list = user.get_all_user_illustrations(user_id)
            illustrations = PixivIllustModel.from_data(data_list)
            count, checked_list = check_files(illustrations, save_path, add_user_folder=True, add_rank=False)[1:3]
            if len(sys.argv) < 2 or count:
                try:
                    print(_('Artists %s [%s]') % (folder, count))
                except UnicodeError:
                    print(_('Artists %s ?? [%s]') % (user_id, count))
            with _PROGRESS_LOCK:
                for index in checked_list:
                    final_list.append(data_list[index])
        except Exception:
            traceback.print_exc()
        user_id_list.task_done()


def update_exist(user, fast=True):
    current_path = get_default_save_path()
    final_list = []
    user_id_list = queue.Queue()
    for folder in os.listdir(current_path):
        if os.path.isdir(os.path.join(current_path, folder)):
            user_id = re.search('^(\d+) ', folder)
            if user_id:
                user_id = user_id.group(1)
                user_id_list.put({'id': user_id, 'folder': folder})
    for i in range(1):
        # use one thread to prevent Rate Limit in new App API
        scan_t = threading.Thread(target=artist_folder_scanner,
                                  args=(user, user_id_list, current_path, final_list, fast,))
        scan_t.daemon = True
        scan_t.start()
    user_id_list.join()
    download_illustrations(user, final_list, current_path, add_user_folder=True)


def remove_repeat(_):
    """Delete xxxxx.img if xxxxx_p0.img exist"""
    choice = input(_('Dangerous Action: continue?(y/n)'))
    if choice == 'y':
        illust_path = get_default_save_path()
        for folder in os.listdir(illust_path):
            if os.path.isdir(os.path.join(illust_path, folder)):
                if re.search('^(\d+) ', folder):
                    path = os.path.join(illust_path, folder)
                    for file_name in os.listdir(path):
                        illustration_id = re.search('^\d+\.', file_name)
                        if illustration_id:
                            if os.path.isfile(os.path.join(path
                                    , illustration_id.string.replace('.', '_p0.'))):
                                os.remove(os.path.join(path, file_name))
                                print('Delete', os.path.join(path, file_name))


def main():
    arguments = docopt(__doc__)
    user = PixivApi()
    if len(sys.argv) > 1:
        print(datetime.datetime.now().strftime('%X %x'))
        ids = arguments['<id>']
        is_rank = arguments['-r']
        date = arguments['--date']
        is_update = arguments['-u']
        if ids:
            download_by_user_id(user, ids)
        elif is_rank:
            if date:
                date = date[0]
                download_by_history_ranking(user, date)
            else:
                download_by_ranking(user)
        elif is_update:
            update_exist(user)
        print(datetime.datetime.now().strftime('%X %x'))
    else:
        print(_(f' Pixiv Downloader {__version__}').center(77, '#'))
        options = {
            '1': download_by_user_id,
            '2': download_by_ranking,
            '3': download_by_history_ranking,
            '4': update_exist,
            '5': remove_repeat
        }
        while True:
            print(_('Which do you want to:'))
            for i in sorted(options.keys()):
                print('\t %s %s' % (i, _(options[i].__name__).replace('_', ' ')))
            choose = input('\t e %s \n:' % _('exit'))
            if choose in [str(i) for i in range(1, len(options) + 1)]:
                print((' ' + _(options[choose].__name__).replace('_', ' ') + ' ').center(60, '#') + '\n')
                if choose == 4:
                    options[choose](user, False)
                else:
                    options[choose](user)
                print('\n' + (' ' + _(options[choose].__name__).replace('_', ' ') + _(' finished ')).center(60,
                                                                                                            '#') + '\n')
            elif choose == 'e':
                break
            else:
                print(_('Wrong input!'))


if __name__ == '__main__':
    sys.exit(main())



import gettext
import os
import sys
import locale

languages = [locale.getdefaultlocale()[0], 'en_US']

current_path = os.path.dirname(os.path.abspath(__file__))
t = gettext.translation('messages', os.path.join(current_path, "locale"), languages=languages)
i18n = t.gettext



#!/usr/bin/env python3
from abc import abstractmethod, ABCMeta


class PixivModel(object):
    """
    store illust/novel data

    Attribute
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def from_data(self, data):
        """parse the data to a dict"""
        pass


class PixivIllustModel(PixivModel):
    @classmethod
    def create_illust_from_data(cls, data):
        illust = cls()
        for k, v in data.items():
            setattr(illust, k, v)
        illust.user_id = str(illust.user['id'])
        illust.user_name = illust.user['name']
        if data['meta_single_page']:
            illust.image_urls = [data['meta_single_page']['original_image_url']]
            if data['type'] == 'ugoira':
                illust.image_urls = [
                    illust.image_urls[0].replace('img-original', 'img-zip-ugoira')
                        .replace('ugoira0.jpg', 'ugoira600x600.zip')
                        .replace('ugoira0.png', 'ugoira600x600.zip')]
        elif data['meta_pages']:
            illust.image_urls = [i['image_urls']['original'] for i in data['meta_pages']]
        return illust

    @classmethod
    def from_data(cls, data_list):
        """parse data to dict contains illust information

        Return:
            result: a list of instance contains illust information
        """
        illusts = []
        for data in list(data_list):
            illust = cls.create_illust_from_data(data)
            illusts.append(illust)
        return illusts



import base64
import uuid

import pyaes

class AESCipher:
    def __init__(self, key=uuid.UUID(int=uuid.getnode()).hex):
        self.key = key.encode('utf-8')

    def encrypt(self, raw):
        cipher = pyaes.AESModeOfOperationCTR(self.key)
        return base64.b64encode(cipher.encrypt(raw))

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        cipher = pyaes.AESModeOfOperationCTR(self.key)
        return cipher.decrypt(enc).decode('utf-8')




import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name="pixivd",
    version="3.0",
    author="KK",
    author_email="bebound@gmail.com",
    description="A simple tool to download illustrations from Pixiv.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bebound/pixiv",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    entry_points={
        'console_scripts': [
            'pixivd=pixivd.pixivd:main',
        ],
    },
    install_requires=[
        'requests>=2.4.3',
        'pyaes>=1.6.1',
        'docopt>=0.6.2',
        'tqdm',
        'PixivPy>=3.6.0',
    ],
    include_package_data=True,
)
